@@ -3,9 +3,11 @@ package Geo::Postcodes;
 use strict;
 use warnings;
 
+our $VERSION = '0.03';
+
 ## Which methods are available ##################################################
 
-my @valid_methods = qw(postcode location borough county type owner address selection);
+my @valid_methods = qw(postcode location borough county type type_verbose owner address selection);
   # Used by the 'methods' method.
 
 my %valid_methods;
@@ -15,19 +17,19 @@ foreach (@valid_methods)
   $valid_methods{$_} = 1;
 }
 
-## Exporter #####################################################################
+## Type Description #############################################################
 
-require Exporter;
-our @ISA = qw(Exporter);
+my %typedesc;
 
-my %EXPORT_TAGS = ( 'all' => [ qw(valid legal location_of borough_of county_of type_of
-                                  owner_of address_of) ] );
+$typedesc{BX}   = "Post Office box";
+$typedesc{ST}   = "Street address";
+$typedesc{SX}   = "Service box";
+$typedesc{IO}   = "Individual owner";
+$typedesc{STBX} = "Street Address and Post Office box";
+$typedesc{MU}   = "Multiple usage";
+$typedesc{PP}   = "Porto Paye receiver";
 
-my @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} });
-
-my @EXPORT = qw();
-
-our $VERSION = '0.02';
+$typedesc{PN}   = "Place name";
 
 ## OO Methods ###################################################################
 
@@ -113,6 +115,15 @@ sub type
   return;
 }
 
+sub type_verbose
+{
+  my $self = shift;
+  return unless defined $self;
+  return unless exists $type_of{$self};
+  return unless exists $typedesc{$type_of{$self}};
+  return $typedesc{$type_of{$self}};
+}
+
 sub owner
 {
   my $self = shift;
@@ -175,6 +186,11 @@ sub type_of
   return;
 }
 
+sub type_verbose_of
+{
+  return;
+}
+
 sub owner_of
 {
   return;
@@ -185,6 +201,19 @@ sub address_of
   return;
 }
 
+sub get_types
+{
+  return keys %typedesc;
+}
+
+sub type2verbose
+{
+  my $type = shift;
+  return unless $type;
+  return unless exists $typedesc{$type};
+  return $typedesc{$type};
+}
+
 sub get_postcodes
 {
   return;
@@ -195,13 +224,12 @@ sub get_postcodes
 
 sub selection
 {
-  return Geo::Postcodes::_selection("Geo::Postcodes", \%valid_methods, @_);
+  return Geo::Postcodes::_selection("Geo::Postcodes", @_);
 }
 
 sub _selection
 {
   my $prefix         = shift;
-  my $legal_methods  = shift; # pointer to hash of legal methods.
   my $oo             = 0; 
   my $method         = shift;
 
@@ -211,17 +239,16 @@ sub _selection
     $method = shift;
   }
 
-  # $prefix .= '::' unless $prefix =~ /::$/;
+  my $is_method = $prefix . "::is_method";
+  my $is_method_pointer = \&{$is_method};
 
-  # my $check = "$prefix::$methods";
-
-  return unless $$legal_methods{$method}; # Check for a valid method
+  return unless &$is_method_pointer($method);
 
   my $value = shift;               # The value to search for
 
   return unless $value;            # A validity check is impossible, so this is the next best thing.
 
-# if ($value =~ /\|/)
+# if ($value =~ /\|/) - should be '$method'
 # {
 #   my @value_fixed;
 #   foreach my $new_value (split /|/, $value)
@@ -282,7 +309,8 @@ Geo::Postcodes - Base class for the Geo::Postcodes::XX modules
 
 This is the base class for the Geo::Postcodes::XX modules.
 
-See the 'SUBCLASS' file for information on how to make your own subclass.
+Note that information on how to make a country specific subclass will not be
+written until the API is finalised.
 
 =head1 ABSTRACT
 
@@ -292,33 +320,168 @@ useless on its own.
 =head1 COMMON FEATURES
 
 The child classes inherit the following methods and procedures (through
-some black magic).
+some black magic):
 
 =head2 selection procedure
 
-my @postcodes = Geo::Postcodes::NO::selection(method => $value);
+ my @postcodes = Geo::Postcodes::XX::selection($method => $string);
 
 Use this to get a list of all the postcodes associated with the
-specified method and value. The methods can be either one mentioned
-in the METHODS section, except I<selection>.
+specified method and string, where 'XX' is substituted by a country
+subclass 'XX'. The methods can be anyone given by the
+C<Geo::Postcodes::XX::methods()> call.
 
-Wildcards are supported, by placing one or more I<%> in $value. The
-I<%> will match zero or more arbitrary characters (as in standard SQL).
-Use I<.> (a single period) to match exactly one character.
+The search string is parsed as the regular expression C<m{^$string$}i>.
+This has the following implications:
 
-DEFECTIVE: Use | to combine searches; Location and I<%S|%SKOG> will
-give all  locations ending with "S" or "SKOG". CAVEAT: This will enter
-an implicit I<%> before and after the |, if not given.
+=over
 
-The match is done case insensitive, but this does not work for the special
-norwegian and danish characters '', '' and ''.
+=item m{...}i
+
+The trailing i means that the search is done case insensitive. This
+does not work for the special norwegian and danish characters '',
+'' and '' unless a C<use locale> is used in the program, and the
+current locale supports these characters. (This does not work when
+the code is running with mod_perl on my web servers, until I find
+a solution.)
+
+I<'As'> will match 'AS', 'As', 'aS', and 'as'.
+
+=item m{^...$}
+
+The first (I<^>; caret) and last (I<$>; dollar sign) character inside
+the expression force a matcth to the whole expression. 
+
+I<'AS'> will match exactly the four variations mentioned above, and
+nothing else (so that 'CHAS' or 'ASIMOV' will not match).
+
+Use L<"Wildcards"> or L<"Regular expressions"> to match several
+characters at once.
+
+=back
+
+=head3 Wildcards
+
+The character I<%> (the percent character) will match zero or more
+arbitrary characters (and is borrowed from standard SQL).
+
+I<'%12'> will match '1112' but not '1201'. I<'O%D'> will match all
+strings starting with an 'O' and ending with a 'D'. I<'%A%'> will match all
+strings with an 'A' somewhere.
+
+(The character I<%> is changed to the regular expression '.*' (dot star)
+by the module.)
+
+=head3 Regular expressions
+
+=over
+
+=item .
+
+The character . (a single dot) will match exactly one character.
+
+I<'..11'> will match a four character string, where the first two can be anything,
+followed by '11'.
+
+=item ?
+
+The character ? (a question mark) will match the previous character
+zero or one time.
+
+I<'%YA?'> will match strings ending with 'Y' or 'YA'.
+
+=item *
+
+The character * (a star) will match the previous character zero or
+more times. 
+
+=item []
+
+The expression I<'[AB]'> will match one of 'A' or'B'.
+
+I<%'[AB]'> will match all names ending with an 'A' or 'B'.
+
+=cut ()
+
+The expression I<'(..)'> will remember the part inside the paranthesis.
+See the next item for usage.
+
+It can also be used in combination with back references; C<\1>, C<\2>
+and so on. I<(..)\1> will match postcodes starting with two caharcters,
+and ending with the same ones (e.g. '1919', 7272', but not '1221').
+I<(.)(.)\2\1> will match postcodes where the first and fourth digit is
+the same, and the second and third digit is the same.
+
+=cut |
+
+The expression I<'A|BBB'> will match either 'A' or 'BBB'.
+
+Be careful with this construct, as I<'%S|%SKOG'> will B<not>
+match '%S' or '%SKOG', but rather everything starting with 'S'
+or ending with 'SKOG' - caused by the C<'^...$'> that the expression
+is wrapped in. Use I<'%(S|SKOG)'> to get the desired result.
 
 =head2 selection method
 
-my @postcodobjects = Geo::Postcodes::NO->selection(method => $value);
+ my @postcodobjects = Geo::Postcodes::XX->selection($method => $string);
 
 This works just as the procedure version (see above), but will return
-a list of postcode objects (instead of a list of postcodes).
+a list of postcode objects (instead of just the postcodes).
+
+=head2 type2verbose procedure
+
+  my $type_as_english_text  = $Geo::Postcodes::type2verbose($type);
+  my $type_as_national_text = $Geo::Postcodes::XX:type2verbose($type);
+
+The child classes are responsible for translating the relevant types.
+
+See the L<"TYPE"> section for a description of the types.
+
+=head1 TYPE
+
+This class defines the following types for the postal locations:
+
+=over
+
+=item BX
+
+Post Office box
+
+=item ST
+
+Street address
+
+=item SX
+
+Service box (as a Post Office box, but the mail is delivered to
+the customer).
+
+=item IO
+
+Individual owner (a company with its own postcode).
+
+=item STBX
+
+Either a Street address (ST) or a Post Office box (BX)
+
+=item MU
+
+Multiple usage (a mix of the other types)
+
+=item PP
+
+Porto Paye receiver (mail where the reicever will pay the postage).
+
+=item PN
+
+Place name
+
+=cut
+
+The child classes can use a subset of these, and define the descriptions
+in the native language if appropriate.
+
+Use C<type2verbose> (see above) to get the description for a given code.
 
 =head1 DESCRIPTION
 
@@ -329,17 +492,15 @@ This module uses "inside out objects".
 =head1 SEE ALSO
 
 The latest version of this library should always be available on CPAN, but see
-also the library home page; L<http://bbop.org/perl/GeoPostcodes> for addittional
+also the library home page; L<http://bbop.org/perl/GeoPostcodes> for additional
 information and sample usage.
 
 =head1 AUTHOR
 
-Arne Sommer, E<lt>arne@cpan.orgE<gt>
+Arne Sommer, E<lt>perl@bbop.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2006 by Arne Sommer
-
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
 
